==> weld-osgi-bundle-3.1.0.Final-sources.jar: org/jboss/weld/injection/FieldResourceInjection.java <==
/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat, Inc., and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.weld.injection;

import static org.jboss.weld.injection.Exceptions.rethrowException;

import java.lang.reflect.Field;
import java.security.AccessController;

import org.jboss.weld.bean.proxy.DecoratorProxy;
import org.jboss.weld.injection.spi.ResourceReferenceFactory;
import org.jboss.weld.interceptor.util.proxy.TargetInstanceProxy;
import org.jboss.weld.security.GetAccessibleCopyOfMember;
import org.jboss.weld.util.reflection.Reflections;

/**
 * High-level representation of a resource field.
 *
 * @author Martin Kouba
 *
 * @param <T>
 * @param <X>
 */
class FieldResourceInjection<T, X> extends AbstractResourceInjection<T> {

    private final Field accessibleField;

    /**
     *
     * @param fieldInjectionPoint
     * @param factory
     */
    FieldResourceInjection(FieldInjectionPoint<T, X> fieldInjectionPoint, ResourceReferenceFactory<T> factory) {
        super(factory);
        this.accessibleField = AccessController.doPrivileged(new GetAccessibleCopyOfMember<Field>(fieldInjectionPoint.getAnnotated().getJavaMember()));
    }

    @Override
    protected void injectMember(Object declaringInstance, Object reference) {
        try {
            Object instanceToInject = declaringInstance;
            if (!(instanceToInject instanceof DecoratorProxy)) {
                // if declaringInstance is a proxy, unwrap it
                if (instanceToInject instanceof TargetInstanceProxy) {
                    instanceToInject = Reflections.<TargetInstanceProxy<T>> cast(declaringInstance).weld_getTargetInstance();
                }
            }
            accessibleField.set(instanceToInject, reference);
        } catch (IllegalArgumentException e) {
            rethrowException(e);
        } catch (IllegalAccessException e) {
            rethrowException(e);
        }
    }

    @Override
    Field getMember() {
        return accessibleField;
    }

}

==> weld-osgi-bundle-5.1.4.Final-sources.jar: org/jboss/weld/injection/FieldResourceInjection.java <==
/*
 * JBoss, Home of Professional Open Source
 * Copyright 2008, Red Hat, Inc., and individual contributors
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.weld.injection;

import static org.jboss.weld.injection.Exceptions.rethrowException;

import java.lang.reflect.Field;
import java.security.AccessController;

import org.jboss.weld.bean.proxy.DecoratorProxy;
import org.jboss.weld.injection.spi.ResourceReferenceFactory;
import org.jboss.weld.interceptor.util.proxy.TargetInstanceProxy;
import org.jboss.weld.security.GetAccessibleCopyOfMember;
import org.jboss.weld.util.reflection.Reflections;

/**
 * High-level representation of a resource field.
 *
 * @author Martin Kouba
 *
 * @param <T>
 * @param <X>
 */
class FieldResourceInjection<T, X> extends AbstractResourceInjection<T> {

    private final Field accessibleField;

    /**
     *
     * @param fieldInjectionPoint
     * @param factory
     */
    FieldResourceInjection(FieldInjectionPoint<T, X> fieldInjectionPoint, ResourceReferenceFactory<T> factory) {
        super(factory);
        this.accessibleField = AccessController
                .doPrivileged(new GetAccessibleCopyOfMember<Field>(fieldInjectionPoint.getAnnotated().getJavaMember()));
    }

    @Override
    protected void injectMember(Object declaringInstance, Object reference) {
        try {
            Object instanceToInject = declaringInstance;
            if (!(instanceToInject instanceof DecoratorProxy)) {
                // if declaringInstance is a proxy, unwrap it
                if (instanceToInject instanceof TargetInstanceProxy) {
                    instanceToInject = Reflections.<TargetInstanceProxy<T>> cast(declaringInstance).weld_getTargetInstance();
                }
            }
            accessibleField.set(instanceToInject, reference);
        } catch (IllegalArgumentException e) {
            rethrowException(e);
        } catch (IllegalAccessException e) {
            rethrowException(e);
        }
    }

    @Override
    Field getMember() {
        return accessibleField;
    }

}
